"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _reactServerModuleTagger = _interopRequireDefault(require("react-server-module-tagger"));

var _path = _interopRequireDefault(require("path"));

var t = _interopRequireWildcard(require("@babel/types"));

module.exports = function () {
  return {
    visitor: {
      Identifier: function Identifier(p, state) {
        var node = p.node;
        var name = node.name;
        var tokens = getTokens(state);

        if (tokens.has(name)) {
          var trim = state.opts.trim;
          var prefix = state.opts.prefix;

          var parent = _path["default"].resolve(_path["default"].join(process.cwd(), '..')) + _path["default"].sep;

          var filePath = this.file.opts.filename.replace(parent, '');
          var options = JSON.parse((0, _reactServerModuleTagger["default"])({
            filePath: filePath,
            trim: trim,
            prefix: prefix
          }));
          p.replaceWith(convertObjectToObjectExpression(options));
        }
      },
      CallExpression: function CallExpression(p, state) {
        var node = p.node;
        var callee = node.callee;
        var name = callee.name;
        var tokens = getTokens(state);

        if (tokens.has(name)) {
          var trim = state.opts.trim;
          var prefix = state.opts.prefix;

          var parent = _path["default"].resolve(_path["default"].join(process.cwd(), '..')) + _path["default"].sep;

          var filePath = this.file.opts.filename.replace(parent, '');
          var opts = {};
          node.arguments[0].properties.forEach(function (prop) {
            opts[prop.key.name] = prop.value.value;
          });
          var options = JSON.parse((0, _reactServerModuleTagger["default"])({
            filePath: filePath,
            trim: trim,
            prefix: prefix,
            opts: opts
          }));
          p.replaceWith(convertObjectToObjectExpression(options));
        }
      }
    }
  };
};

exports["default"] = module.exports;

function convertObjectToObjectExpression(obj) {
  var properties = [];
  Object.keys(obj).forEach(function (key) {
    var literal;

    switch ((0, _typeof2["default"])(obj[key])) {
      case 'string':
        literal = t.stringLiteral(obj[key]);
        break;

      case 'number':
        literal = t.numericLiteral(obj[key]);
        break;

      case 'object':
        literal = convertObjectToObjectExpression(obj[key]);
        break;

      default:
        throw new Error("got unexpected objecy property type of ".concat((0, _typeof2["default"])(obj[key])));
    }

    properties.push(t.objectProperty(t.identifier(key), literal));
  });
  return t.objectExpression(properties);
}

function getTokens(state) {
  if (state.opts.tokens) {
    return new Set(state.opts.tokens);
  } else {
    return new Set(["__LOGGER__", "__CHANNEL__", "__CACHE__"]);
  }
}